@@ -223,10 +223,10 @@ gth_main_initialize (void)
 	Main = (GthMain*) g_object_new (GTH_TYPE_MAIN, NULL);
 
 #ifdef GDK_WINDOWING_X11
-	egg_set_desktop_file (GTHUMB_DATADIR "/applications/gthumb.desktop");
+	egg_set_desktop_file (GTHUMB_APPLICATIONS_DIR "/gthumb.desktop");
 #else
 	/* manually set name and icon */
-	g_set_application_name (_("gthumb"));
+	g_set_application_name (_("gThumb"));
 	gtk_window_set_default_icon_name ("gthumb");
 #endif
 
@@ -404,6 +404,8 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 
 	for (i = 0; metadata_info[i].id != NULL; i++)
 		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL)) {
+			if (metadata_info[i].sort_order <= 0)
+				metadata_info[i].sort_order = 500;
 			g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
 			g_hash_table_insert (Main->priv->metadata_info_hash, (gpointer) (&metadata_info[i])->id, &metadata_info[i]);
 		}
@@ -1184,7 +1186,7 @@ gth_main_tags_changed (void)
 GthMonitor *
 gth_main_get_default_monitor (void)
 {
-	if (Main->priv->monitor != NULL)
+	if (G_LIKELY (Main->priv->monitor != NULL))
 		return Main->priv->monitor;
 
 	Main->priv->monitor = gth_monitor_new ();
@@ -1206,7 +1208,8 @@ gth_main_activate_extensions (void)
 	const char *mandatory_extensions[] = {	"file_viewer",
 						"cairo_io",
 						NULL };
-	const char *default_extensions[] = {	"bookmarks",
+	const char *default_extensions[] = {	"23hq",
+						"bookmarks",
 						"burn_disc",
 						"catalogs",
 						"change_date",
